@@ -19,14 +19,16 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import requests, urllib.request, urllib.parse, urllib.error, platform, threading, tempfile, traceback
-import os, sys
+import urllib.request, urllib.parse, urllib.error, platform, threading, traceback
+import sys
 from xml.etree.ElementTree import XML
 from . import Device
 from . import _debugprint
 
 __all__ = ['OpenPrinting']
 
+HTTPS_TIMEOUT = 15.0
+
 def _normalize_space (text):
     result = text.strip ()
     result = result.replace ('\n', ' ')
@@ -64,17 +66,15 @@ class _QueryThread (threading.Thread):
                    self.parent.language[0]))
         self.url = "https://%s%s?%s" % (self.parent.base_url, query_command, params)
         # Send request
-        result = None
         self.result = b''
         status = 1
         try:
-            req = requests.get(self.url, verify=True)
-            self.result = req.content
-            status = 0
-        except:
-            self.result = sys.exc_info ()
-            if status is None:
+            req = urllib.request.Request(self.url, headers=headers)
+            with urllib.request.urlopen(req, timeout=HTTPS_TIMEOUT) as resp:
+                self.result = resp.read()
                 status = 0
+        except:
+            self.result = sys.exc_info()
 
         _debugprint ("%s: query complete" % self)
         if self.callback is not None:
@@ -287,9 +287,8 @@ class OpenPrinting:
                             license_url = element.text
                             if license_url is not None:
                                 try:
-                                    req = requests.get(license_url, verify=True)
-                                    dict['licensetext'] = \
-                                        req.content.decode("utf-8")
+                                    with urllib.request.urlopen(license_url, timeout=HTTPS_TIMEOUT) as resp:
+                                        dict['licensetext'] = resp.read().decode('utf-8')
                                 except:
                                     _debugprint('Cannot retrieve %s' %
                                                 license_url)
@@ -338,7 +337,7 @@ class OpenPrinting:
                     packages = {}
                     container = driver.find ('packages')
                     if container is not None:
-                        for arch in container.getchildren ():
+                        for arch in list(container):
                             rpms = {}
                             for package in arch.findall ('package'):
                                 rpm = {}
@@ -351,7 +350,7 @@ class OpenPrinting:
 
                                 repositories = package.find ('repositories')
                                 if repositories is not None:
-                                    for pkgsys in repositories.getchildren ():
+                                    for pkgsys in list(repositories):
                                         rpm.setdefault('repositories', {})[pkgsys.tag] = pkgsys.text
 
                                 rpms[package.attrib['file']] = rpm
@@ -363,7 +362,7 @@ class OpenPrinting:
                     ppds = []
                     container = driver.find ('ppds')
                     if container is not None:
-                        for each in container.getchildren ():
+                        for each in list(container):
                             ppds.append (each.text)
 
                     if ppds:
