@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
@@ -28,7 +28,7 @@ import time
 from debug import *
 import pprint
 import gettext
-gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 import ppdcache
 import statereason
 from statereason import StateReason
@@ -52,51 +52,51 @@ def collect_printer_state_reasons (connection, ppdcache):
     except cups.IPPError:
         return result
 
-    for name, printer in printers.iteritems ():
+    for name, printer in printers.items ():
         reasons = printer["printer-state-reasons"]
         for reason in reasons:
             if reason == "none":
                 break
             if state_reason_is_harmless (reason):
                 continue
-            if not result.has_key (name):
+            if name not in result:
                 result[name] = []
             result[name].append (StateReason (name, reason, ppdcache))
     return result
 
 class Monitor(GObject.GObject):
     __gsignals__ = {
-        'refresh':               (GObject.SIGNAL_RUN_LAST, None, ()),
-        'monitor-exited' :       (GObject.SIGNAL_RUN_LAST, None, ()),
-        'state-reason-added':    (GObject.SIGNAL_RUN_LAST, None,
+        'refresh':               (GObject.SignalFlags.RUN_LAST, None, ()),
+        'monitor-exited' :       (GObject.SignalFlags.RUN_LAST, None, ()),
+        'state-reason-added':    (GObject.SignalFlags.RUN_LAST, None,
                                   (GObject.TYPE_PYOBJECT,)),
-        'state-reason-removed':  (GObject.SIGNAL_RUN_LAST, None,
+        'state-reason-removed':  (GObject.SignalFlags.RUN_LAST, None,
                                   (GObject.TYPE_PYOBJECT,)),
-        'still-connecting':      (GObject.SIGNAL_RUN_LAST, None,
+        'still-connecting':      (GObject.SignalFlags.RUN_LAST, None,
                                   (GObject.TYPE_PYOBJECT,)),
-        'now-connected':         (GObject.SIGNAL_RUN_LAST, None,
+        'now-connected':         (GObject.SignalFlags.RUN_LAST, None,
                                   (str,)),
-        'job-added':             (GObject.SIGNAL_RUN_LAST, None,
+        'job-added':             (GObject.SignalFlags.RUN_LAST, None,
                                   (int, str,
                                    GObject.TYPE_PYOBJECT,
                                    GObject.TYPE_PYOBJECT,)),
-        'job-event':             (GObject.SIGNAL_RUN_LAST, None,
+        'job-event':             (GObject.SignalFlags.RUN_LAST, None,
                                   (int, str,
                                    GObject.TYPE_PYOBJECT,
                                    GObject.TYPE_PYOBJECT,)),
-        'job-removed':           (GObject.SIGNAL_RUN_LAST, None,
+        'job-removed':           (GObject.SignalFlags.RUN_LAST, None,
                                   (int, str,
                                    GObject.TYPE_PYOBJECT,)),
-        'printer-added':         (GObject.SIGNAL_RUN_LAST, None,
+        'printer-added':         (GObject.SignalFlags.RUN_LAST, None,
                                   (str,)),
-        'printer-event':         (GObject.SIGNAL_RUN_LAST, None,
+        'printer-event':         (GObject.SignalFlags.RUN_LAST, None,
                                   (str, str,
                                    GObject.TYPE_PYOBJECT,)),
-        'printer-removed':       (GObject.SIGNAL_RUN_LAST, None,
+        'printer-removed':       (GObject.SignalFlags.RUN_LAST, None,
                                   (str,)),
-        'cups-connection-error': (GObject.SIGNAL_RUN_LAST, None, ()),
-        'cups-connection-recovered': (GObject.SIGNAL_RUN_LAST, None, ()),
-        'cups-ipp-error':        (GObject.SIGNAL_RUN_LAST, None,
+        'cups-connection-error': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'cups-connection-recovered': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'cups-ipp-error':        (GObject.SignalFlags.RUN_LAST, None,
                                   (int, str,))
         }
 
@@ -140,7 +140,7 @@ class Monitor(GObject.GObject):
         self.received_any_dbus_signals = False
         self.update_timer = None
 
-        if bus == None:
+        if bus is None:
             try:
                 bus = dbus.SystemBus ()
             except dbus.exceptions.DBusException:
@@ -148,7 +148,7 @@ class Monitor(GObject.GObject):
                 pass
 
         self.bus = bus
-        if bus != None:
+        if bus is not None:
             bus.add_signal_receiver (self.handle_dbus_signal,
                                      path=self.DBUS_PATH,
                                      dbus_interface=self.DBUS_IFACE)
@@ -177,12 +177,12 @@ class Monitor(GObject.GObject):
                 pass
             cups.setUser (user)
 
-        if self.bus != None:
+        if self.bus is not None:
             self.bus.remove_signal_receiver (self.handle_dbus_signal,
                                              path=self.DBUS_PATH,
                                              dbus_interface=self.DBUS_IFACE)
 
-        timers = self.connecting_timers.values ()
+        timers = list(self.connecting_timers.values ())
         for timer in [self.update_timer, self.fetch_jobs_timer]:
             if timer:
                 timers.append (timer)
@@ -203,7 +203,7 @@ class Monitor(GObject.GObject):
             self.connecting_timers[printer] = timer
             return False
 
-        if self.connecting_timers.has_key (printer):
+        if printer in self.connecting_timers:
             del self.connecting_timers[printer]
 
         debugprint ("Still-connecting timer fired for `%s'" % printer)
@@ -218,13 +218,12 @@ class Monitor(GObject.GObject):
         time_now = time.time ()
         connecting_to_device = {}
         trouble = False
-        for printer, reasons in self.printer_state_reasons.iteritems ():
+        for printer, reasons in self.printer_state_reasons.items ():
             connected = True
             for reason in reasons:
                 if reason.get_reason () == "connecting-to-device":
                     have_processing_job = False
-                    for job, data in \
-                            printer_jobs.get (printer, {}).iteritems ():
+                    for job, data in printer_jobs.get (printer, {}).items ():
                         state = data.get ('job-state',
                                           cups.IPP_JOB_CANCELED)
                         if state == cups.IPP_JOB_PROCESSING:
@@ -246,7 +245,7 @@ class Monitor(GObject.GObject):
                             if printer not in self.still_connecting:
                                 self.still_connecting.add (printer)
                                 self.emit ('still-connecting', reason)
-                            if self.connecting_timers.has_key (printer):
+                            if printer in self.connecting_timers:
                                 GLib.source_remove (self.connecting_timers
                                                     [printer])
                                 del self.connecting_timers[printer]
@@ -256,7 +255,7 @@ class Monitor(GObject.GObject):
                     connected = False
                     break
 
-            if connected and self.connecting_timers.has_key (printer):
+            if connected and printer in self.connecting_timers:
                 GLib.source_remove (self.connecting_timers[printer])
                 del self.connecting_timers[printer]
                 debugprint ("Stopped connecting timer for `%s'" % printer)
@@ -264,10 +263,10 @@ class Monitor(GObject.GObject):
         # Clear any previously-notified errors that are now fine.
         remove = set()
         for printer in self.still_connecting:
-            if not connecting_to_device.has_key (printer):
+            if printer not in connecting_to_device:
                 remove.add (printer)
                 self.emit ('now-connected', printer)
-                if self.connecting_timers.has_key (printer):
+                if printer in self.connecting_timers:
                     GLib.source_remove (self.connecting_timers[printer])
                     del self.connecting_timers[printer]
                     debugprint ("Stopped connecting timer for `%s'" % printer)
@@ -277,14 +276,14 @@ class Monitor(GObject.GObject):
 
     def check_state_reasons(self, my_printers=set(), printer_jobs={}):
         # Look for any new reasons since we last checked.
-        old_reasons_seen_keys = self.reasons_seen.keys ()
+        old_reasons_seen_keys = list(self.reasons_seen.keys ())
         reasons_now = set()
-        for printer, reasons in self.printer_state_reasons.iteritems ():
+        for printer, reasons in self.printer_state_reasons.items ():
             for reason in reasons:
                 tuple = reason.get_tuple ()
                 printer = reason.get_printer ()
                 reasons_now.add (tuple)
-                if not self.reasons_seen.has_key (tuple):
+                if tuple not in self.reasons_seen:
                     # New reason.
                     GLib.idle_add (lambda x:
                                        self.emit ('state-reason-added', x),
@@ -292,12 +291,11 @@ class Monitor(GObject.GObject):
                     self.reasons_seen[tuple] = reason
 
                 if (reason.get_reason () == "connecting-to-device" and
-                    not self.connecting_to_device.has_key (printer)):
+                    printer not in self.connecting_to_device):
                     # First time we've seen this.
 
                     have_processing_job = False
-                    for job, data in \
-                            printer_jobs.get (printer, {}).iteritems ():
+                    for job, data in printer_jobs.get (printer, {}).items ():
                         state = data.get ('job-state',
                                           cups.IPP_JOB_CANCELED)
                         if state == cups.IPP_JOB_PROCESSING:
@@ -319,7 +317,7 @@ class Monitor(GObject.GObject):
                             debugprint (pprint.pformat (printer_jobs))
 
         self.update_connecting_devices (printer_jobs)
-        items = self.reasons_seen.keys ()
+        items = list(self.reasons_seen.keys ())
         for tuple in items:
             if not tuple in reasons_now:
                 # Reason no longer present.
@@ -400,14 +398,14 @@ class Monitor(GObject.GObject):
 
                 elif nse == 'printer-deleted' and name in self.printers:
                     self.printers.remove (name)
-                    items = self.reasons_seen.keys ()
+                    items = list(self.reasons_seen.keys ())
                     for tuple in items:
                         if tuple[1] == name:
                             reason = self.reasons_seen[tuple]
                             del self.reasons_seen[tuple]
                             self.emit ('state-reason-removed', reason)
                             
-                    if self.printer_state_reasons.has_key (name):
+                    if name in self.printer_state_reasons:
                         del self.printer_state_reasons[name]
 
                     self.emit ('printer-removed', name)
@@ -436,9 +434,9 @@ class Monitor(GObject.GObject):
             jobid = event['notify-job-id']
             if (nse == 'job-created' or
                 (nse == 'job-state-changed' and
-                 not jobs.has_key (jobid) and
+                 jobid not in jobs and
                  event['job-state'] == cups.IPP_JOB_PROCESSING)):
-                if (self.specific_dests != None and
+                if (self.specific_dests is not None and
                     event['printer-name'] not in self.specific_dests):
                     continue
 
@@ -473,7 +471,7 @@ class Monitor(GObject.GObject):
             except KeyError:
                 continue
 
-            if (self.specific_dests != None and
+            if (self.specific_dests is not None and
                 event['printer-name'] not in self.specific_dests):
                 del jobs[jobid]
                 self.emit ('job-removed', jobid, nse, event)
@@ -482,7 +480,7 @@ class Monitor(GObject.GObject):
             for attribute in ['job-state',
                               'job-name']:
                 job[attribute] = event[attribute]
-            if event.has_key ('notify-printer-uri'):
+            if 'notify-printer-uri' in event:
                 job['job-printer-uri'] = event['notify-printer-uri']
 
             self.emit ('job-event', jobid, nse, event, job.copy ())
@@ -507,7 +505,7 @@ class Monitor(GObject.GObject):
         debugprint ("refresh")
 
         self.emit ('refresh')
-        if which_jobs != None:
+        if which_jobs is not None:
             self.which_jobs = which_jobs
 
         user = cups.getUser ()
@@ -526,9 +524,8 @@ class Monitor(GObject.GObject):
                 c.cancelSubscription (self.sub_id)
             except cups.IPPError as e:
                 (e, m) = e.args
-                GLib.idle_add (lambda (e, m):
-                                   self.emit ('cups-ipp-error', e, m),
-                               (e, m))
+                GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                               e, m)
 
             if self.update_timer:
                 GLib.source_remove (self.update_timer)
@@ -558,9 +555,8 @@ class Monitor(GObject.GObject):
                                                                 repr (events)))
         except cups.IPPError as e:
             (e, m) = e.args
-            GLib.idle_add (lambda (e, m):
-                               self.emit ('cups-ipp-error', e, m),
-                           (e, m))
+            GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                           e, m)
 
         cups.setUser (user)
 
@@ -579,7 +575,7 @@ class Monitor(GObject.GObject):
             if self.which_jobs not in ['all', 'completed']:
                 # Filter out completed jobs.
                 filtered = {}
-                for jobid, job in jobs.iteritems ():
+                for jobid, job in jobs.items ():
                     if job.get ('job-state',
                                 cups.IPP_JOB_CANCELED) < cups.IPP_JOB_CANCELED:
                         filtered[jobid] = job
@@ -600,15 +596,14 @@ class Monitor(GObject.GObject):
             self.printers = set(dests.keys ())
         except cups.IPPError as e:
             (e, m) = e.args
-            GLib.idle_add (lambda (e, m):
-                               self.emit ('cups-ipp-error', e, m),
-                           (e, m))
+            GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                           e, m)
             return
         except RuntimeError:
             GLib.idle_add (self.emit, 'cups-connection-error')
             return
 
-        if self.specific_dests != None:
+        if self.specific_dests is not None:
             for jobid in jobs.keys ():
                 uri = jobs[jobid].get('job-printer-uri', '/')
                 i = uri.rfind ('/')
@@ -619,10 +614,10 @@ class Monitor(GObject.GObject):
         self.set_process_pending (False)
         for printer in self.printers:
             GLib.idle_add (lambda x: self.emit ('printer-added', x), printer)
-        for jobid, job in jobs.iteritems ():
-            GLib.idle_add (lambda (jobid, job):
-                               self.emit ('job-added', jobid, '', {}, job),
-                           (jobid, job))
+        for jobid, job in jobs.items ():
+            GLib.idle_add (lambda jobid, job:
+                                  self.emit ('job-added', jobid, '', {}, job),
+                           jobid, job)
         self.update_jobs (jobs)
         self.jobs = jobs
         self.set_process_pending (True)
@@ -671,7 +666,7 @@ class Monitor(GObject.GObject):
         debugprint ("Got %s jobs, asked for %s" % (got, limit))
 
         jobs = self.jobs.copy ()
-        jobids = fetched.keys ()
+        jobids = list(fetched.keys ())
         jobids.sort ()
         if got > 0:
             last_jobid = jobids[got - 1]
@@ -681,17 +676,17 @@ class Monitor(GObject.GObject):
                 debugprint ("That's not what we asked for!")
         else:
             last_jobid = self.fetch_first_job_id + limit - 1
-        for jobid in xrange (self.fetch_first_job_id, last_jobid + 1):
+        for jobid in range (self.fetch_first_job_id, last_jobid + 1):
             try:
                 job = fetched[jobid]
-                if self.specific_dests != None:
+                if self.specific_dests is not None:
                     uri = job.get('job-printer-uri', '/')
                     i = uri.rfind ('/')
                     printer = uri[i + 1:]
                     if printer not in self.specific_dests:
                         raise KeyError
 
-                if jobs.has_key (jobid):
+                if jobid in jobs:
                     n = 'job-event'
                 else:
                     n = 'job-added'
@@ -700,11 +695,11 @@ class Monitor(GObject.GObject):
                 self.emit (n, jobid, '', {}, job.copy ())
             except KeyError:
                 # No job by that ID.
-                if jobs.has_key (jobid):
+                if jobid in jobs:
                     del jobs[jobid]
                     self.emit ('job-removed', jobid, '', {})
 
-        jobids = jobs.keys ()
+        jobids = list(jobs.keys ())
         jobids.sort ()
         if got < limit:
             trim = False
@@ -728,19 +723,19 @@ class Monitor(GObject.GObject):
         # Remember where we got up to and run this timer again.
         next = jobid + 1
 
-        while not refresh_all and self.jobs.has_key (next):
+        while not refresh_all and next in self.jobs:
             next += 1
 
         self.fetch_first_job_id = next
         return True
 
     def sort_jobs_by_printer (self, jobs=None):
-        if jobs == None:
+        if jobs is None:
             jobs = self.jobs
 
         my_printers = set()
         printer_jobs = {}
-        for job, data in jobs.iteritems ():
+        for job, data in jobs.items ():
             state = data.get ('job-state', cups.IPP_JOB_CANCELED)
             if state >= cups.IPP_JOB_CANCELED:
                 continue
@@ -750,7 +745,7 @@ class Monitor(GObject.GObject):
                 continue
             printer = uri[i + 1:]
             my_printers.add (printer)
-            if not printer_jobs.has_key (printer):
+            if printer not in printer_jobs:
                 printer_jobs[printer] = {}
             printer_jobs[printer][job] = data
 
@@ -794,46 +789,43 @@ if __name__ == '__main__':
             monitor.connect ('cups-ipp-error', self.on_cups_ipp_error)
 
         def on_monitor_exited (self, obj):
-            print "*%s: monitor exited" % obj
+            print("*%s: monitor exited" % obj)
 
         def on_state_reason_added (self, obj, reason):
-            print "*%s: +%s" % (obj, reason)
+            print("*%s: +%s" % (obj, reason))
 
         def on_state_reason_removed (self, obj, reason):
-            print "*%s: -%s" % (obj, reason)
+            print("*%s: -%s" % (obj, reason))
 
         def on_still_connecting (self, obj, reason):
-            print "*%s: still connecting: %s" % (obj, reason)
+            print("*%s: still connecting: %s" % (obj, reason))
 
         def on_now_connected (self, obj, printer):
-            print "*%s: now connected: %s" % (obj, printer)
+            print("*%s: now connected: %s" % (obj, printer))
 
         def on_job_added (self, obj, jobid, eventname, event, jobdata):
-            print "*%s: job %d added" % (obj, jobid)
+            print("*%s: job %d added" % (obj, jobid))
 
         def on_job_event (self, obj, jobid, eventname, event, jobdata):
-            print "*%s: job %d event: %s" % (obj, jobid, event)
+            print("*%s: job %d event: %s" % (obj, jobid, event))
 
         def on_job_removed (self, obj, jobid, eventname, event):
-            print "*%s: job %d removed (%s)"% (obj, jobid, eventname)
+            print("*%s: job %d removed (%s)"% (obj, jobid, eventname))
 
         def on_printer_added (self, obj, name):
-            print "*%s: printer added: %s" % (obj, name)
+            print("*%s: printer added: %s" % (obj, name))
 
         def on_printer_event (self, obj, name, eventname, event):
-            print "*%s: printer event: %s: %s" % (obj, name, eventname)
+            print("*%s: printer event: %s: %s" % (obj, name, eventname))
 
         def on_printer_removed (self, obj, name):
-            print "*%s: printer %s removed" % (obj, name)
+            print("*%s: printer %s removed" % (obj, name))
 
         def on_cups_connection_error (self, obj):
-            print "*%s: cups connection error" % obj
+            print("*%s: cups connection error" % obj)
 
         def on_cups_ipp_error (self, obj, err, errstring):
-            # cups.IPPError message is (utf-8) encoded
-            if isinstance(errstring, bytes):
-                    errstring = errstring.decode('utf-8', 'replace')
-            print "*%s: IPP error (%d): %s" % (obj, err, errstring)
+            print("*%s: IPP error (%d): %s" % (obj, err, errstring))
 
     set_debugging (True)
     m = Monitor ()
