@@ -1,6 +1,6 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
-## Copyright (C) 2006, 2007, 2008, 2010, 2012 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2010, 2012, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -41,21 +41,23 @@ class UserDefaultPrinter:
             return
 
         try:
-            opts = file (self.lpoptions).readlines ()
+            opt_file = open(self.lpoptions)
+            opts = opt_file.readlines ()
         except IOError:
             return
 
         for i in range (len (opts)):
             if opts[i].startswith ("Default "):
                 opts[i] = "Dest " + opts[i][8:]
-        file (self.lpoptions, "w").writelines (opts)
+        open (self.lpoptions, "w").writelines (opts)
 
     def get (self):
         if not self.lpoptions:
             return None
 
         try:
-            opts = file (self.lpoptions).readlines ()
+            opt_file = open(self.lpoptions)
+            opts = opt_file.readlines ()
         except IOError:
             return None
 
@@ -73,13 +75,13 @@ class UserDefaultPrinter:
     def set (self, default):
         p = subprocess.Popen ([ "lpoptions", "-d", default ],
                               close_fds=True,
-                              stdin=file ("/dev/null"),
-                              stdout=file ("/dev/null", "w"),
+                              stdin=subprocess.DEVNULL,
+                              stdout=subprocess.DEVNULL,
                               stderr=subprocess.PIPE)
         (stdout, stderr) = p.communicate ()
         exitcode = p.wait ()
         if exitcode != 0:
-            raise RuntimeError (exitcode, stderr.strip ())
+            raise RuntimeError (exitcode, stderr.decode ().strip ())
         return
 
     def __repr__ (self):
@@ -138,7 +140,7 @@ class UserDefaultPrompt:
         systemwide.set_active (True)
         clearpersonal.set_active (True)
         self.userdef = UserDefaultPrinter ()
-        clearpersonal.set_sensitive (self.userdef.get () != None)
+        clearpersonal.set_sensitive (self.userdef.get () is not None)
 
         self.systemwide = systemwide
         self.clearpersonal = clearpersonal
@@ -148,7 +150,7 @@ class UserDefaultPrompt:
         dialog.show_all ()
 
     def on_toggled (self, button):
-        self.clearpersonal.set_sensitive (self.userdef.get () != None and
+        self.clearpersonal.set_sensitive (self.userdef.get () is not None and
                                           self.systemwide.get_active ())
 
     def on_response (self, dialog, response_id):
@@ -164,7 +166,7 @@ class UserDefaultPrompt:
             try:
                 self.userdef.set (self.name)
             except Exception as e:
-                print "Error setting default: %s" % repr (e)
+                print("Error setting default: %s" % repr (e))
 
             self.refresh_fn ()
 
